/*
 * Copyright 2005-2007 Freescale Semiconductor, Inc. All Rights Reserved.
 */

/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*=============================================================================
                                                                               
    Module Name:    mfw_gst_m2vparser.c

    General Description:    Implementation of  M2V Parser Plugin                      
                            
===============================================================================
Portability:    This code is written for Linux OS and Gstreamer

  This code is only for testing purposes. It is not of release quality.

===============================================================================
                            INCLUDE FILES
=============================================================================*/
#include <gst/gst.h>
#include <gst/base/gstadapter.h>
#include "mfw_gst_m2vparser.h"


/*=============================================================================
                            LOCAL CONSTANTS
=============================================================================*/

/*=============================================================================
                LOCAL TYPEDEFS (STRUCTURES, UNIONS, ENUMS)
=============================================================================*/

enum
{
    ID_0,
    ID_FRAMERATE
};


/*=============================================================================
                                        LOCAL MACROS
=============================================================================*/
/* used for debugging */
#define GST_CAT_DEFAULT mfw_gst_m2vparser_debug


/*=============================================================================
                                      LOCAL VARIABLES
=============================================================================*/
/******************************************************************************                                                            
         source pad  properties of m2vparser element                     
******************************************************************************/
static GstStaticPadTemplate mfw_gst_m2vparser_src_template_factory =
GST_STATIC_PAD_TEMPLATE("src",
			GST_PAD_SRC,
			GST_PAD_ALWAYS,
			GST_STATIC_CAPS("video/mpeg, "
					"mpegversion = (int) [1, 2 ], "
					"systemstream = (boolean) false")
    );

/******************************************************************************
                sink pad properties of m2vparser element                  
******************************************************************************/
static GstStaticPadTemplate mfw_gst_m2vparser_sink_template_factory =
GST_STATIC_PAD_TEMPLATE("sink",
			GST_PAD_SINK,
			GST_PAD_ALWAYS,
			GST_STATIC_CAPS("video/mpeg, "
					"mpegversion = (int) [1 ,2 ] , "
					"systemstream = (boolean) false")
    );

/*                details of m2vparser element                              */
static GstElementDetails mfw_gst_m2vparser_details =
		    GST_ELEMENT_DETAILS( "dummy plugin ",
		    "NULL  ",
		    "M2V Parser Plugin",
		    "Multimedia Team <mmsw@freescale.com>" );

/* used in change state function */
 static GstElementClass *parent_class = NULL;  

 
/*=============================================================================
                        LOCAL FUNCTION PROTOTYPES
============================================================================*/
                                            
GST_DEBUG_CATEGORY_STATIC                           (mfw_gst_m2vparser_debug );
static GType   mfw_gst_m2vparser_get_type           (void );
static void mfw_gst_m2vparser_base_init             (gpointer );
static void mfw_gst_m2vparser_class_init            (MFW_GST_M2VPARSER_INFO_CLASS_T* );
static void mfw_gst_m2vparser_init                  (MFW_GST_M2VPARSER_INFO_T* );
static gboolean  mfw_gst_m2vparser_set_caps         (GstPad* , GstCaps* );
static gboolean mfw_gst_m2vparser_sink_event        (GstPad* , GstEvent* );
static GstStateChangeReturn 
mfw_gst_m2vparser_change_state                      (GstElement* ,GstStateChange );
static GstFlowReturn  mfw_gst_m2vparser_chain       ( GstPad *, GstBuffer *);


static void mfw_gst_m2vparser_get_value             (guint8* );
static void
mfw_gst_m2vparser_get_property (GObject *object, guint prop_id,
                                  GValue *value, GParamSpec *pspec);
static void
mfw_gst_m2vparser_set_property (GObject *object, guint prop_id,
                                  const GValue *value, GParamSpec *pspec);


/*=============================================================================
                            GLOBAL VARIABLES
=============================================================================*/



/*=============================================================================
                            LOCAL FUNCTIONS
=============================================================================*/
/*=============================================================================
FUNCTION: mfw_gst_m2vparser_set_property

DESCRIPTION: sets the property of the element

ARGUMENTS PASSED:
        object     - pointer to the elements object
        prop_id    - ID of the property;
        value      - value of the property set by the application
        pspec      - pointer to the attributes of the property

RETURN VALUE:
        None

PRE-CONDITIONS:
        None

POST-CONDITIONS:


IMPORTANT NOTES:
        None
=============================================================================*/

static void
mfw_gst_m2vparser_set_property (GObject *object, guint prop_id,
                                  const GValue *value, GParamSpec *pspec)
{
  GST_DEBUG(" in mfw_gst_m2vparser_set_property routine \n");
  MFW_GST_M2VPARSER_INFO_T *m2vparser_info = MFW_GST_M2VPARSER(object);
  switch (prop_id)
  {
  case ID_FRAMERATE:
    m2vparser_info->framerate = g_value_get_float (value);
    break;
  default:
    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
    break;
  }
  GST_DEBUG(" out of mfw_gst_m2vparser_set_property routine \n");
}

/*=============================================================================
FUNCTION: mfw_gst_m2vparser_get_property

DESCRIPTION: gets the property of the element

ARGUMENTS PASSED:
        object     - pointer to the elements object
        prop_id    - ID of the property;
        value      - value of the property to be set for the next element
        pspec      - pointer to the attributes of the property

RETURN VALUE:
        None

PRE-CONDITIONS:
        None

POST-CONDITIONS:


IMPORTANT NOTES:
        None
=============================================================================*/
static void
mfw_gst_m2vparser_get_property (GObject *object, guint prop_id,
                                  GValue *value, GParamSpec *pspec)
{
  GST_DEBUG(" in mfw_gst_m2vparser_get_property routine \n");
  MFW_GST_M2VPARSER_INFO_T *m2vparser_info = MFW_GST_M2VPARSER(object);
  switch (prop_id) {
  case ID_FRAMERATE:
    g_value_set_float(value, m2vparser_info->framerate);
    break;
  default:
    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
    break;
  }
  GST_DEBUG(" out of mfw_gst_m2vparser_get_property routine \n");
}

/*=============================================================================

FUNCTION:   mfw_gst_m2vparser_chain

DESCRIPTION:    main function which does parsing .

ARGUMENTS PASSED:
            buffer ---  pointer to the GstBuffer
			pad    ---- pointer to pad

RETURN VALUE:
           GstFlowReturn

PRE-CONDITIONS:
        None

POST-CONDITIONS:
		None

IMPORTANT NOTES:
        None

====================================================================================*/
static GstFlowReturn 
mfw_gst_m2vparser_chain(GstPad * pad,
					   GstBuffer * buffer)
{

    GstAdapter *adapter = NULL;
    guint8* data=NULL;
    gint32 size=0;
    gint32 temp=0;
    gint32 i_loop=0; 
    GstBuffer* push_buffer=NULL;
    GstFlowReturn  result   = GST_FLOW_OK; 
    guint8 *data_push=NULL;
	guint8 *data_out=NULL;
    GstBuffer* dump=NULL; 
	GstCaps *src_caps = NULL;
	GstBuffer* outbuffer=NULL;
	GstCaps *caps = NULL;
	MFW_GST_M2VPARSER_INFO_T *m2vparser_info;

	
	m2vparser_info = MFW_GST_M2VPARSER(GST_OBJECT_PARENT(pad));  
    m2vparser_info->offset =0; 

    GST_DEBUG(" coming in mfw_gst_m2vparser_chain \n");
  
    
    adapter = (GstAdapter *) m2vparser_info->adapter;
	/* Use only valid timestamp sent by the demuxer */
    if( GST_BUFFER_TIMESTAMP_IS_VALID (buffer))
    {
		GST_DEBUG ( "receiving timestamp = %" GST_TIME_FORMAT, GST_TIME_ARGS(GST_BUFFER_TIMESTAMP(buffer)));	
        if(m2vparser_info->loop_chain<2)
		{
			/* Storing the first two timestamps to calculate time duration b/w two timestamps */ 
			m2vparser_info->timestamp_buffer[ m2vparser_info->timestamp_tx]=GST_BUFFER_TIMESTAMP(buffer);
			m2vparser_info->timestamp_org=m2vparser_info->timestamp_buffer[ m2vparser_info->timestamp_tx];
			if( m2vparser_info->loop_chain==1)
   			{
	     		m2vparser_info->timestamp_duration=m2vparser_info->timestamp_buffer[ m2vparser_info->timestamp_tx]-m2vparser_info->timestamp_buffer[ m2vparser_info->timestamp_tx-1] ;
				m2vparser_info->timestamp_org=m2vparser_info->timestamp_org- m2vparser_info->timestamp_duration;
				
			}
			m2vparser_info->timestamp_tx =(m2vparser_info->timestamp_tx+1)%MAX_STREAM_BUF ;
			m2vparser_info->loop_chain++;
		}
		
	}
	/* push buffer into adapter */ 
	gst_adapter_push(adapter, buffer);
   	size = gst_adapter_available(adapter);
	data = gst_adapter_peek(adapter, size);

	if (FALSE == m2vparser_info->caps_set) 
	{
		if(	m2vparser_info->frame_rate_de  == 0)
		{
			m2vparser_info->frame_rate_nu=0;
			m2vparser_info->frame_rate_de =1;
		}
		/* passing frame rate to mpeg2decoder */ 
		caps = gst_caps_new_simple("video/mpeg",
							"framerate", GST_TYPE_FRACTION, 
							m2vparser_info->frame_rate_nu , m2vparser_info->frame_rate_de ,
							NULL);
			
		if (!(gst_pad_set_caps(m2vparser_info->srcpad, caps))) 
		{
			GST_DEBUG (" Could not set the caps for the decoder src pad \n");
		}
		
		gst_caps_unref(caps);
		m2vparser_info->caps_set= TRUE;
	
	}
    
	src_caps = GST_PAD_CAPS(m2vparser_info->srcpad);

    while(i_loop < size)
    {       
	
        if((data[i_loop]==0x00)&&(data[i_loop+1]==0x00)&&(data[i_loop+2]==0x01)&&(data[i_loop+3]==0x00))
        {
                       
			m2vparser_info->offset=m2vparser_info->offset+4;
			/* start code found */
            i_loop=i_loop+4;
           
			if( TRUE == m2vparser_info->first_time)
			{       
				if(size <= m2vparser_info->offset)
				{
					GST_DEBUG(" input size is less than offset \n");
					m2vparser_info->offset=0;
					break ; 
				}
         

				if(m2vparser_info->flag ==FALSE)
				{
					/* pull out required buffer from adapter */ 
					push_buffer= gst_adapter_take_buffer(adapter, m2vparser_info->offset-4);
					dump = gst_adapter_take_buffer(adapter, 4);            
					m2vparser_info->flag  =TRUE;
            
				}
				else
				{
					GST_DEBUG("FRAME SIZE = %d\n" , m2vparser_info->offset) ; 
					/* pull out required buffer from adapter */ 
					push_buffer= gst_adapter_take_buffer(adapter, m2vparser_info->offset-4);
					dump = gst_adapter_take_buffer(adapter,4); 
                    push_buffer=gst_buffer_join(dump,push_buffer);

				}
                            
			result = gst_pad_alloc_buffer_and_set_caps(m2vparser_info->srcpad,
						   0, GST_BUFFER_SIZE(push_buffer),
						   src_caps,
						   &outbuffer);
					
			if (GST_FLOW_OK != result) 
			{
				GST_DEBUG(" not able to allocate buffer \n");
				return result;
			}

			data_push=GST_BUFFER_DATA(push_buffer);
			data_out=GST_BUFFER_DATA(outbuffer);
			
			memcpy(data_out,data_push,GST_BUFFER_SIZE(push_buffer));
				
			m2vparser_info->timestamp_org+=m2vparser_info->timestamp_duration;
		
			GST_BUFFER_TIMESTAMP(outbuffer) =  m2vparser_info->timestamp_org;   

			GST_DEBUG ( "pushing timestamp = %" GST_TIME_FORMAT, GST_TIME_ARGS(GST_BUFFER_TIMESTAMP(outbuffer)));	    	

			result = gst_pad_push(m2vparser_info->srcpad,outbuffer); 
	
			if (GST_FLOW_OK != result) 
			{
				GST_DEBUG(" not able to push data  \n");
				return result;
				
			}
		      
			gst_buffer_unref(push_buffer);

                        size= size-m2vparser_info->offset;

			m2vparser_info->offset=0;
            
			}
    
        m2vparser_info->first_time=TRUE;
    
        }
        else
        {
        m2vparser_info->offset++;
        i_loop++;
        
        }
        
    }
  
    GST_DEBUG(" going out mfw_gst_m2vparser_chain \n");
    return GST_FLOW_OK;

}

/*=============================================================================
FUNCTION:   mfw_gst_m2vparser_sink_event

DESCRIPTION:    send an event to sink pad of m2vparser element

ARGUMENTS PASSED:
        pad        -    pointer to pad
        event      -    pointer to event
RETURN VALUE:
        TRUE       -	if event is sent to sink properly
	    FALSE	   -	if event is not sent to sink properly

PRE-CONDITIONS:
        None

POST-CONDITIONS:
        None

IMPORTANT NOTES:
        None
=============================================================================*/
static gboolean 
mfw_gst_m2vparser_sink_event (GstPad* pad, GstEvent* event)
{
	MFW_GST_M2VPARSER_INFO_T* m2vparser_struct  = MFW_GST_M2VPARSER
                                                    (GST_PAD_PARENT (pad));
	gboolean result			                    = 1;
    guint32 buf_size                            = 0;
    GstBuffer *buff                             = NULL;
	GstFlowReturn  res   = GST_FLOW_OK; 
    GST_DEBUG(" in mfw_gst_m2vparser_sink_event function \n");
	GST_DEBUG ("handling %s event", GST_EVENT_TYPE_NAME (event));
	
    switch (GST_EVENT_TYPE (event)) 
	{
		case GST_EVENT_NEWSEGMENT:
		{
			GstFormat format;
			gst_event_parse_new_segment (event, NULL, NULL, &format, NULL, \
                                         NULL,NULL);
			if (format == GST_FORMAT_TIME) 
			{
					
				result = gst_pad_push_event (m2vparser_struct->srcpad, event);
			
			}
			else 
			{
				GST_DEBUG ("dropping  new segment event in format %s",
				            gst_format_get_name (format));
			    	gst_event_unref (event);
				result = TRUE;
			}
			break;
		}

		case GST_EVENT_EOS:
        {
            
			result  = gst_pad_push_event (m2vparser_struct->srcpad, event);

            if(result != TRUE)
			{
				GST_ERROR("\n Error in pushing the event, result is %d\n", \
                          result);
				
			}
			else
            {
         		GST_DEBUG("\n EOS event sent to the peer element\n");
            }
            break;
        }
        default:
        {    		
            result = gst_pad_event_default (pad, event);
	
            break;
        }	
        
     }
	
    return result;
}


/*=============================================================================
FUNCTION:   mfw_gst_m2vparser_set_caps

DESCRIPTION:    this function handles the link with other plug-ins and used for
                capability negotiation  between pads

ARGUMENTS PASSED:
        pad        -    pointer to GstPad
        caps       -    pointer to GstCaps

RETURN VALUE:
        TRUE       -    if capabilities are set properly
        FALSE      -    if capabilities are not set properly
PRE-CONDITIONS:
        None

POST-CONDITIONS:
        None

IMPORTANT NOTES:
        None
=============================================================================*/
static gboolean 
mfw_gst_m2vparser_set_caps (GstPad *pad, GstCaps *caps)
{
	
    GST_DEBUG(" int mfw_gst_m2vparser_set_cap function \n");
   

    MFW_GST_M2VPARSER_INFO_T *m2vparser_struct  ;
	const gchar* mime;
    GstStructure *structure     = gst_caps_get_structure(caps, 0);
    
  
	m2vparser_struct            = MFW_GST_M2VPARSER (gst_pad_get_parent (pad));
	mime                        = gst_structure_get_name (structure);
 
	if (strcmp (mime, "video/mpeg") !=	0)
	{
		GST_WARNING("Wrong mimetype %s	provided, we only support %s",
				 mime, "video/mpeg");
		return FALSE;
	}

    gst_structure_get_fraction(structure, "framerate", &m2vparser_struct->frame_rate_nu,
			       &m2vparser_struct->frame_rate_de);
	
	GST_DEBUG(" m2vparser_struct->frame_rate_nu = %d \n", m2vparser_struct->frame_rate_nu);
	GST_DEBUG(" m2vparser_struct->frame_rate_de = %d \n", m2vparser_struct->frame_rate_de);

	GST_DEBUG(" Frame Rate = %f \n", m2vparser_struct->frame_rate);


    if(!gst_pad_set_caps(m2vparser_struct->srcpad, caps))
	{
		return FALSE;
	}
	
    GST_DEBUG(" out of mfw_gst_m2vparser_set_caps function \n");

    return TRUE;
}



/*=============================================================================
FUNCTION:   mfw_gst_m2vparser_init

DESCRIPTION:    create the pad template that has been registered with the 
                element class in the _base_init and do library table 
                initialization

ARGUMENTS PASSED:
        m2vparser_struct  -    pointer to m2vparser element structure

RETURN VALUE:
        None

PRE-CONDITIONS:
        _base_init and _class_init are called 

POST-CONDITIONS:
        None

IMPORTANT NOTES:
        None
=============================================================================*/
static void
mfw_gst_m2vparser_init (MFW_GST_M2VPARSER_INFO_T * m2vparser_struct  )
{
    GST_DEBUG("\n in mfw_gst_m2vparser_init  routine\n"); 
  
    
    GstPadTemplate *template;
    GstElementClass *klass      = GST_ELEMENT_GET_CLASS(m2vparser_struct  );
    
    /* create the sink and src pads */
    m2vparser_struct->sinkpad = gst_pad_new_from_template (
                                    gst_element_class_get_pad_template(
                                        klass, "sink"), "sink");
    gst_element_add_pad (GST_ELEMENT (m2vparser_struct  ), m2vparser_struct  ->sinkpad);
    gst_pad_set_setcaps_function (m2vparser_struct->sinkpad,
                                    mfw_gst_m2vparser_set_caps);
    gst_pad_set_event_function (m2vparser_struct->sinkpad,GST_DEBUG_FUNCPTR (
                                    mfw_gst_m2vparser_sink_event));
    m2vparser_struct->srcpad = gst_pad_new_from_template (
                                    gst_element_class_get_pad_template(
                                        klass, "src"), "src");
    gst_element_add_pad (GST_ELEMENT (m2vparser_struct  ), m2vparser_struct->srcpad);
  
    gst_pad_set_chain_function(m2vparser_struct->sinkpad,
			       GST_DEBUG_FUNCPTR(mfw_gst_m2vparser_chain));
    
    m2vparser_struct->caps_set = FALSE;
    m2vparser_struct->framerate = 0.0;
  
    GST_DEBUG(" out of mfw_gst_m2vparser_init routine\n");
}



/*=============================================================================
FUNCTION:   mfw_gst_m2vparser_class_init

DESCRIPTION:    Initialise the class only once (specifying what signals,
                arguments and virtual functions the class has and setting up 
                global state)    

ARGUMENTS PASSED:
       	klass   -   pointer to m2vparser element class

RETURN VALUE:
        None

PRE-CONDITIONS:
        None

POST-CONDITIONS:
        None

IMPORTANT NOTES:
        None
=============================================================================*/
static void 
mfw_gst_m2vparser_class_init (MFW_GST_M2VPARSER_INFO_CLASS_T * klass)
{
	GST_DEBUG(" in mfw_gst_m2vparser_class_init function \n");

    GObjectClass *gobject_class		    = NULL;
	GstElementClass	*gstelement_class   = NULL;
	gobject_class				        = (GObjectClass *) klass;
	gstelement_class			        = (GstElementClass *) klass;
    parent_class                        = g_type_class_ref (GST_TYPE_ELEMENT);
    gstelement_class->change_state      = mfw_gst_m2vparser_change_state;
    gobject_class->set_property		    = mfw_gst_m2vparser_set_property;
    gobject_class->get_property		    = mfw_gst_m2vparser_get_property;

    g_object_class_install_property (gobject_class, ID_FRAMERATE,
        g_param_spec_float("framerate", "FrameRate", "gets the framerate at which the input stream is to be displayed",
        -G_MAXFLOAT,G_MAXFLOAT, 0.0, G_PARAM_READWRITE));

  
    GST_DEBUG(" out of mfw_gst_m2vparser_class_init function \n");

}



/*=============================================================================
FUNCTION:  mfw_gst_m2vparser_base_init	

DESCRIPTION:    m2vparser element details are registered with the plugin during
                _base_init ,This function will initialise the class and child 
                class properties during each struct child class creation     

ARGUMENTS PASSED:
        Klass   -   void pointer

RETURN VALUE:
        None

PRE-CONDITIONS:
        None

POST-CONDITIONS:
        None

IMPORTANT NOTES:
        None
=============================================================================*/
static void 
mfw_gst_m2vparser_base_init(gpointer klass)
{
  	GST_DEBUG(" in mfw_gst_m2vparser_base_init function \n");
  
    GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
	gst_element_class_add_pad_template (element_class, 
	    gst_static_pad_template_get (&mfw_gst_m2vparser_sink_template_factory));
  	gst_element_class_add_pad_template (element_class, 
	    gst_static_pad_template_get (&mfw_gst_m2vparser_src_template_factory));
	gst_element_class_set_details (element_class, &mfw_gst_m2vparser_details);
    
    GST_DEBUG(" out of mfw_gst_m2vparser_base_init function \n");
}


/*=============================================================================
FUNCTION:   mfw_gst_m2vparser_change_state

DESCRIPTION: this function keeps track of different states of pipeline.        
    
ARGUMENTS PASSED:
        element     -   pointer to element 
        transition  -   state of the pipeline

RETURN VALUE:
        GST_STATE_CHANGE_FAILURE    - the state change failed  
        GST_STATE_CHANGE_SUCCESS    - the state change succeeded  
        GST_STATE_CHANGE_ASYNC      - the state change will happen 
                                        asynchronously  
        GST_STATE_CHANGE_NO_PREROLL - the state change cannot be prerolled  

PRE-CONDITIONS:
        None

POST-CONDITIONS:
        None

IMPORTANT NOTES:
        None
=============================================================================*/        
static GstStateChangeReturn
mfw_gst_m2vparser_change_state(GstElement * element, GstStateChange transition)
{
    MFW_GST_M2VPARSER_INFO_T *m2vparser_struct;
    GstStateChangeReturn ret;
    m2vparser_struct=MFW_GST_M2VPARSER(element);
    switch (transition) 
    {
        case GST_STATE_CHANGE_NULL_TO_READY:
        {
            m2vparser_struct->caps_set          = FALSE;
            m2vparser_struct->offset            = 0;
            m2vparser_struct->total_duration    = 0;
            m2vparser_struct->buffstart         = 0;
            m2vparser_struct->new_seg_flag      = 0;
            m2vparser_struct->first_time =FALSE;
            m2vparser_struct->flag=FALSE;
            
            break;
        }
        case GST_STATE_CHANGE_READY_TO_PAUSED:
        {    
               m2vparser_struct->adapter = gst_adapter_new();
          
            break;
        }
        case  GST_STATE_CHANGE_PAUSED_TO_PLAYING:
            break;
        default:
            break;
    }

    ret = parent_class->change_state (element, transition);

    switch (transition) 
    {
        case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
            break;
        case GST_STATE_CHANGE_PAUSED_TO_READY:
			
			gst_adapter_clear(m2vparser_struct->adapter);
			g_object_unref(m2vparser_struct->adapter);

			m2vparser_struct->adapter = NULL;
            break;
        case GST_STATE_CHANGE_READY_TO_NULL:
            break;
        default:
            break;
    }
    return ret;
}


/*=============================================================================
FUNCTION:   mfw_gst_m2vparser_get_type

DESCRIPTION:    intefaces are initiated in this function.you can register one 
                or more interfaces  after having registered the type itself.

ARGUMENTS PASSED:
            None

RETURN VALUE:
            A numerical value ,which represents the unique identifier of this
            element(m2vparser)

PRE-CONDITIONS:
            None

POST-CONDITIONS:
            None

IMPORTANT NOTES:
            None
=============================================================================*/
GType
mfw_gst_m2vparser_get_type(void)
{
   
    
    static GType m2vparser_type = 0;

    if (!m2vparser_type)
    {
        static const GTypeInfo gstm2vparser_info =
        {
            sizeof (MFW_GST_M2VPARSER_INFO_CLASS_T),
            mfw_gst_m2vparser_base_init,
            NULL,
            (GClassInitFunc) mfw_gst_m2vparser_class_init,
            NULL,
            NULL,
            sizeof (MFW_GST_M2VPARSER_INFO_T),
            0,
            (GInstanceInitFunc) mfw_gst_m2vparser_init,
        };
        m2vparser_type = g_type_register_static (GST_TYPE_ELEMENT,
                                                    "MFW_GST_M2VPARSER_INFO_T",
                                                     &gstm2vparser_info, 0);
    }

    GST_DEBUG_CATEGORY_INIT (mfw_gst_m2vparser_debug, "mfw_m2vparser", 
                             0,"FreeScale's dummy plugin's  Log");

    return m2vparser_type;
}


/*=============================================================================
FUNCTION:   plugin_init

DESCRIPTION:    special function , which is called as soon as the plugin or 
                element is loaded and information returned by this function 
                will be cached in central registry

ARGUMENTS PASSED:
        plugin     -    pointer to container that contains features loaded 
                        from shared object module

RETURN VALUE:
        return TRUE or FALSE depending on whether it loaded initialized any 
        dependency correctly

PRE-CONDITIONS:
        None

POST-CONDITIONS:
        None

IMPORTANT NOTES:
        None
=============================================================================*/
static gboolean
plugin_init (GstPlugin * plugin)
{
    return gst_element_register (plugin, "mfw_m2vparser",GST_RANK_PRIMARY,
                                    MFW_GST_TYPE_M2VPARSER);
}




/*===========================================================================*/


/*****************************************************************************/
/*    This is used to define the entry point and meta data of plugin         */
/*****************************************************************************/
GST_PLUGIN_DEFINE (
        GST_VERSION_MAJOR,                  /* major version of Gstreamer    */
        GST_VERSION_MINOR,                  /* minor version of Gstreamer    */      
        "mfw_m2vparser",                    /* name of the plugin            */ 
        "parses the m2v file  ",            /* what plugin actually does     */
        plugin_init,                        /* first function to be called   */
        VERSION,                                
        GST_LICENSE_UNKNOWN,                
        "freescale semiconductor",
        "www.freescale.com "
           )

                                                               

